unit test;

interface

Uses Classes, Hash;

Type TAction = (A_Create, A_Destroy, A_Get, A_Set, A_IsKey, A_RemoveKey, A_Nothing);

Type TTest = Class

        Private

        // vstupni parametry
        action : TAction;
        size : Integer; // pro Create
        key : String;
        value : String;

        // spravny vysledek
        exception : Boolean;
        rvalue : String;
        bvalue : Boolean;

        Public

        Constructor Create;
        // vraci False v pripade chyby
        Function setup(line : String) : Boolean;
        Function test(Var h : TStringArray) : Boolean;

        Private

        Procedure clear;
        // rozdeli vstupni retezec na podretece oddelene mezerami
        Procedure parse(s : String; a : TStringList);

     End;

Function is_string(s : String) : Boolean;
Function get_string(s : String) : String;

implementation

uses SysUtils;

Function is_string(s : String) : Boolean;
Begin
  is_string := False;

  If Length(s) < 2 Then Exit;
  If (s[1] <> '''') Or (s[Length(s)] <> '''') Then Exit;

  is_string := True;
End;


Function get_string(s : String) : String;
Begin
  get_string := Copy(s, 2, Length(s) - 2);
End;


Constructor TTest.Create;
Begin
  Inherited Create;
  clear;
End;


Function TTest.setup(line : String) : Boolean;
Var list : TStringList;
    s : String;
Begin
  list := Nil;   
  Try
    clear;
    setup := False;

    list := TStringList.Create;
    parse(line, list);


    If list[0] = 'Create' Then
           Begin
             action := A_Create;
             size := StrToInt(list[1]);
             If list.Count <> 2 Then Exit;
           End
    Else If list[0] = 'Destroy' Then
           Begin
             action := A_Destroy;
             If list.Count <> 1 Then Exit;
           End
    Else If list[0] = 'Get' Then
           Begin
             action := A_Get;
             key := get_string(list[1]);
             s := list[2];
             If is_string(s) Then
               rvalue := get_string(s)
             Else
               If s = 'exception' Then exception := True
             Else
               Exit;
             If list.Count <> 3 Then Exit;
           End
    Else If list[0] = 'Set' Then
           Begin
             action := A_Set;
             key := get_string(list[1]);
             value := get_string(list[2]);
             If list.Count <> 3 Then Exit;
           End
    Else If list[0] = 'IsKey' Then
           Begin
             action := A_IsKey;
             key := get_string(list[1]);
             s := list[2];
             If s = 'True' Then
                bvalue := True
             Else
               If s <> 'False' Then Exit;
             If list.Count <> 3 Then Exit;
           End
    Else If list[0] = 'RemoveKey' Then
           Begin
             action := A_RemoveKey;
             key := get_string(list[1]);
             s := list[2];
             If s = 'exception' Then
                exception := True
             Else
               If s <> 'ok' Then Exit;
             If list.Count <> 3 Then Exit;
           End
    Else
      Exit;
  Finally
    list.Free;
  End;

  setup := True;
End;


Function TTest.test(Var h : TStringArray) : Boolean;
Begin
  test := True;

  Case action Of

    A_Create: Begin
        h := TStringArray.Create(size);
      End;
    A_Destroy: Begin
        h.Destroy;
        h := Nil;
      End;

    A_Get: Begin
        Try
          If rvalue <> h.Data[key] Then test := False;
          If exception Then test := False;
        Except
          On EInvalidKey Do If Not exception Then test := False;
        End;
      End;

    A_Set: Begin
        Try
          h.Data[key] := value;
          If exception Then test := False;
        Except
          On EInvalidKey Do If Not exception Then test := False;
        End;
      End;
    A_IsKey: If bvalue <> h.IsKey(key) Then test := False;
    A_RemoveKey: Begin
        Try
          h.RemoveKey(key);
          If exception Then test := False;
        Except
          On EInvalidKey Do If Not exception Then test := False;
        End;
      End;
  End;
End;


Procedure TTest.clear;
Begin
  action := A_Nothing;
  size := 0;
  key := '';
  value := '';

  exception := False;
  rvalue := '';
  bvalue := False;
End;


Procedure TTest.parse(s : String; a : TStringList);
Var p : Integer;
Begin
  While s <> '' Do
    Begin
      p := Pos(' ', s);
      If p = 0 Then p := Length(s) + 1;
      a.Add(Copy(s, 1, p - 1));
      s := Copy(s, p + 1, Length(s));
    End;
End;

end.
